package orderprocessing;

import java.util.Enumeration;
import java.util.Hashtable;
import java.io.Serializable;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;

/** This class maintains information about a single customer

	History
	date		description
	-----		-----------
  11/12/99	Modifed from the original, create separate address class
  12/13/99  Creation of inner class DBMaintenance.  Also, each constructor
            of Customer puts customer object into DB after created.
  12/15/99  Added methods that provide means of changing customer addresses.
            Also added the ability to "remove" a customer and print all
            customers.

*/
public class Customer implements Serializable, Versionable
{	/** create a customer object with default values for ship address and terms
	*/
   public Customer(String name, Address billingAddress)
	{	this(name, billingAddress, billingAddress, BillingTerms.NET_30_DAYS);
	}

	/** Create a customer object with default terms.
	*/
   public Customer(String name, Address billingAddress, Address shipAddress)
	{	// To Do: error checking for id and name
	    this.name      			= name;
	    this.billingAddress		= billingAddress;
	    this.shipAddress 		= shipAddress;

      allCustomers.put( key, this );
	}

	/** create a customer object with all options specified.
	* @param name the customer name.
	* @param billingAddress the customer billing address
	* @param shipAddress the ship address if not specified on an order
	* @param billingTerms the billing terms if not specified on an order.
	*/
    public Customer
    (	String name,
    	Address billingAddress,
    	Address shipAddress,
    	BillingTerms billingTerms
    )
	{	this.name       		= name;
	    this.billingAddress		= billingAddress;
	    this.shipAddress 		= shipAddress;
	    this.terms 				= billingTerms;

      allCustomers.put( key, this );
	}

	public Object newVersion(Object information){return null; }
	
	public void display()
	{	System.out.println(this);
	}

	public boolean equals(Object other)
	{	if(!(other instanceof Customer))return false;
		return key.equals(((Customer)other).key);
	}

	public int hashcode()
	{	return key.hashCode()*997;
	}

	// Accessors
	public String  key()        { return key;      }
	public String  name()       { return name;    }
	public Address billingAddress()    { return billingAddress; }

  /** Only change the name of customer of billing address
  * @param name  Name of customer to be changed
  */
  public void billingAddress( String name ) {
    billingAddress.name ( name );
  }

  /** Change Billing Address of this customer object.
  * @param name  Customer who receives the bills
  * @param street  Billing address Street
  * @param city  Billing address City
  * @param state  Billing address State
  * @param zip  Billing address Zip
  */
  public void billingAddress( String name, String street, String city,
                              String state, String zip ) 
 {   billingAddress.street( street );
    billingAddress.city( city );
    billingAddress.state( state );
    billingAddress.zip( zip );
  }
	public Address shippingAddress()    { return shipAddress; }

  /** Only change the name of customer of shipping address
  * @param name  Name of customer to be changed
  */
  public void shippingAddress( String name ) {
    shipAddress.name ( name );
  }

  /** Change Shipping Address of this customer object.
  * @param name  Customer who receives the bills
  * @param street  Shipping address Street
  * @param city  Shipping address City
  * @param state  Shipping address State
  * @param zip  Shipping address Zip
  */
  public void shippingAddress( String name, String street, String city,
                              String state, String zip ) 
  {  shipAddress.name( name );
    shipAddress.street( street );
    shipAddress.city( city );
    shipAddress.state( state );
    shipAddress.zip( zip );
  }
	public BillingTerms billingTerms() { return terms;}
	// ToDo's: Mutators
	public void billingTerms(BillingTerms terms){this.terms = terms;}

	public void name(String name)
	{	this.name = name;
		billingAddress.name(name); //avoid redundancy.
	}

	public String toString()
	{	return "\t\t\t\t\t\t" +
	        " Customer ID :" + key   + "\n" +
	        " Name    : " + name    + "\n" +
	        " Address : " + billingAddress + "\n" +
          " Ship Address : " + shipAddress  + "\n " + terms + "\n";
	}

	private String					key  = IDGenerator.nextID();
	private String					name;
	private Address					billingAddress;
	private Address					shipAddress;
	private BillingTerms			terms = BillingTerms.NET_30_DAYS;
    private boolean isActive = true;

  /** This is the inner class that generates an ID in certain order.
  */
	public static class IDGenerator
	{	public static String nextID()
	    {	 return ID_PREFIX + Long.toString(nextID++);
	    }

	    private static long             nextID   = 5001;
	    private final static String     ID_PREFIX = "C";
	}

  /** The database is implemented as a Hashtable which is maintained by
  *   the inner class DBMaintenance.
  */
  private static Hashtable allCustomers = new Hashtable();
  
  public static final void close(ObjectOutputStream out)
	throws IOException
	{	out.writeLong(IDGenerator.nextID);
		out.writeObject(allCustomers);	
	}

   public static final void open(ObjectInputStream in) 
	    throws IOException, ClassNotFoundException
	{	IDGenerator.nextID = in.readLong();
		allCustomers = (Hashtable)in.readObject();
	}

//---------------------------------------------------------------------------
   
   	 public static Enumeration customers() 
     { return allCustomers.elements();
     }

    /** The customer object isn't really removed.  It is just marked as
    *   "inactive".
    * @param ID  Key for searching customer object in the hashtable.
    */
    public static final Object remove( String ID ) 
    { Customer customer = (Customer)allCustomers.get( ID );
      if(customer != null) customer.isActive = false;
      return customer;
    }

    /** Print, without any parameter, will print out the entire list of customers.
    */
    public static void displayAll() 
    { System.out.println( "\n--------------- LISTING OF ALL CUSTOMERS" +
                            "---------------");
      Enumeration e = allCustomers.elements();
      while (e.hasMoreElements()) 
      { Customer customer = (Customer)e.nextElement();
        if ( customer.isActive ) customer.display();
      }
      System.out.println( "\n----------------------------------------" +
                            "---------------");
    }

    /** Print the customer information matched the customer ID.
    * @param ID  String which is also the key for fetching the customer object
    *            in the hashtable.
    */
    public static boolean display(String ID) 
    { Customer customer = (Customer)allCustomers.get( ID );
      if(customer == null) return false;
      if ( customer.isActive )
        customer.display();
      else
        System.out.println( "\n\t\t" + "THIS CUSTOMER IS INACTIVE\n" );
      return true;  // for now
    }

    /** Fetch is exactly equivalent to get for hashtable.
    * @param ID  Key for searching and getting the customer object.
    */
    public static Customer get(String ID) // may be null
    { return (Customer)allCustomers.get( ID );
    }

    /** For now, this is the only way to activate a customer object again
    *   either after it was checked out or "removed".
    */
    public static boolean checkin(String ID) 
    { Customer customer = (Customer)allCustomers.get( ID );
    	 if (customer == null) return false;
      customer.isActive = true;
      return true;  
    }
}