package orderprocessing;

import java.io.*;
import java.util.Enumeration;
import java.util.StringTokenizer;

/** This class accepts and fulfills customer database maintenance requests.

    History
    Date      Description
    --------  -----------
    12/12/99  Accept customer transactions.
    12/14/99  Generate new customer.
    12/14/99  Print each customer.
    12/15/99  Edit customer, remove, print all customers.
*/

public class CustomerProcessor
{	/** Create a new CustomerProcessor for a given Customer Database.

	/** Dispatch transaction command such as print, remove,...
  * @param transaction String with action word and customer ID if necessary
	*/
	public boolean newTransaction ( String transaction	)
	{
    if (transaction.indexOf("print") >= 0)
    {	printCustomer( transaction ); 
    	return true;
    }
    else if (transaction.indexOf("remove") >= 0)
      return removeCustomer( transaction );
    else
      return false;
	}

  /** Generate an order object from string data sent to it
  * @param transaction String with action word and customer ID if necessary
	* @param customer String with name, address, city... tilde ("~") separated
	* @param extraInfo The shipping address and billing terms, if any
	* Note that this is a Factory Method (pattern) for Customer objects
	*/
  public boolean newTransaction (String transaction, String customer,
                                 String extraInfo) {
    if (transaction.indexOf("new") >= 0)
      genCustomer(customer, extraInfo);
    else if (transaction.indexOf("edit") >= 0) {
      StringTokenizer tok = new StringTokenizer( transaction, " " );
      tok.nextToken();
      String ID = tok.nextToken();
      editCustomer( ID, customer, extraInfo );
    }
    return true;  //  for now
  }

  /** Create a new customer object based on info received.
	* @param customer String with name, address, city... tilde ("~") separated
  * @param extraInfo String with ship address, city... and billing terms.
	*/
	private Customer genCustomer(String customer, String extraInfo)
	{
    StringTokenizer tok = new StringTokenizer(customer, "~");
	  String name = tok.nextToken();
	  Address billingAddress = new Address ( name,    // name
		                     			  tok.nextToken(),    // street
		                     			  tok.nextToken(),    // city
		                     			  tok.nextToken(),    // state
		                     			  tok.nextToken()     // zip
		                     			);
    StringTokenizer tok2 = new StringTokenizer(extraInfo, "~");
    String shipTo = "";
    if (tok2.hasMoreTokens())
      shipTo = tok2.nextToken();
    else
      return new Customer(name, billingAddress);
    String terms = "";
		BillingTerms billingTerms = BillingTerms.NET_30_DAYS;
    Address shippingAddress;

    if ((shipTo.indexOf("NET") == 0) || (shipTo.indexOf("EOM") == 0)) {
      terms = shipTo;
      shippingAddress = billingAddress;
    } else {
      shippingAddress = new Address ( shipTo,    // name
		                        			  tok2.nextToken(),    // street
		                        			  tok2.nextToken(),    // city
		                        			  tok2.nextToken(),    // state
		                        			  tok2.nextToken()	  // zip
		                        			);

		  if (tok2.hasMoreTokens()) terms = tok2.nextToken();
    }

    if ( terms.equals("") )
      return new Customer(name, billingAddress, shippingAddress);
    else {
      StringTokenizer tok3 = new StringTokenizer(terms, " ");
      String billType = tok3.nextToken();
      int billDays = Integer.parseInt(tok3.nextToken());

		  if(billType.indexOf("NET") >= 0)
        billingTerms = new BillingTerms(BillingTerms.NET,billDays);
		  else
        billingTerms = new BillingTerms(BillingTerms.EOM, 0);

      return new Customer(name, billingAddress, shippingAddress, billingTerms);
    }
	}

  /** Change existing customer info.
  * @param ID String contains key for retrieving customer object.
	* @param customer String with name, address, city... tilde ("~") separated
  * @param extraInfo String with ship address, city... and billing terms.
	*/
  private boolean editCustomer( String ID, String custInfo, String extraInfo ) {
    Customer customer = (Customer)Customer.get( ID );

    if ( custInfo.indexOf("~") != 0 ) { // Changing Name
      StringTokenizer tok = new StringTokenizer( custInfo, "~" );
      String name = tok.nextToken();

      customer.name( name );

      if ( tok.hasMoreElements() ) { // Changing Billing Address
        customer.billingAddress ( name );  // remember address
      } else
        customer.billingAddress ( name );
    }
    String terms = "";
    if ( (extraInfo.indexOf("NET") == 0) || (extraInfo.indexOf("EOM") == 0))
    // No change of Shipping Address
      terms = extraInfo;
    else {
      StringTokenizer tok2 = new StringTokenizer( extraInfo, "~" );
      customer.shippingAddress( tok2.nextToken(),    // name
		                        		tok2.nextToken(),    // street
		                        		tok2.nextToken(),    // city
		                        		tok2.nextToken(),    // state
		                        		tok2.nextToken()	  // zip
                              );
      if ( tok2.hasMoreTokens() ) terms = tok2.nextToken();
    }
    if ( !terms.equals( "" ) ) {
      StringTokenizer tok3 = new StringTokenizer(terms, " ");
      String billType = tok3.nextToken();
      int billDays = Integer.parseInt(tok3.nextToken());

		  if(billType.indexOf("NET") >= 0)
        customer.billingTerms( new BillingTerms(BillingTerms.NET, billDays) );
		  else
       customer.billingTerms( new BillingTerms(BillingTerms.EOM, 0) );
    }
    return true;  // for now
  }

  /** Print customer info.
  * @param transaction Basically contains "print" and optional customer ID.
	*/
  private void printCustomer( String transaction ) {
    StringTokenizer tok = new StringTokenizer(transaction, " ");
    String ID = "";
    tok.nextToken();
    if (tok.hasMoreTokens()) ID = tok.nextToken();

    if ( ID.equals( "" ) )
      Customer.displayAll();
    else
      Customer.display( ID );
  }

  /** "Remove" customer from database by just mark it inactive.
  * @param transaction Basically contains "remove" and a customer ID.
	*/
  private boolean removeCustomer( String transaction ) {
    StringTokenizer tok = new StringTokenizer( transaction, " " );
    tok.nextToken();
    String ID = tok.nextToken();
    Customer.remove( ID );
    return true;  // for now
  }

  /** Dump database into a file
  */
  public static void close() 
  {  OrderProcessor.close();
  }

  /** Open the database file and deserialize the objects into an input
  *   stream.
  */
  public static void open() 
  {  OrderProcessor.open();
  }

	public static CustomerProcessor customerProcessor = new CustomerProcessor();
}


