package orderprocessing;

/*
	History	
	date		description
	-----		-----------
	12/06/99	Modified to include new maintenance function.	            
*/


import java.util.*;
import java.io.*;

/** This class represents a single order from a single customer.
* Note that this class implements the Immutable Object design pattern as it has
* no mutators for any of its instance variables. 
*/
public class Order implements Serializable, Versionable
{	
	/** Create an Order for a particular customer and a sequence of items ordered
	* @param who The customer ordering these items
	* @param what A vector of OrderItems representing the items ordered
	* @param when The date that this order was placed
	* @param where The Shipping Address if different from the customer's default
	* @param requestedShippingDate Requested shipping date by customer
	* @param cancelDate Requested cancelation date for this order. 
	*/
	public Order	
	(	Customer customer, 
		Address shipAddress,
		BillingTerms billingTerms,
		Vector orderedItems, 
		CalculationCalendar orderDate,
		CalculationCalendar requestedShippingDate,
		CalculationCalendar cancelDate
   )
	{	this.key            = IDGenerator.nextID();
	 	this.customer       = customer;
		this.shipAddress    = shipAddress;
		this.orderedItems   = (Vector) orderedItems.clone();
		this.orderDate      = orderDate;
		this.terms          = billingTerms;
		
		if(this.terms == null) this.terms = customer.billingTerms();
		
		CalculationCalendar earlyShipDate = this.orderDate.incrementedBy(CalculationCalendar.DATE, MIN_SHIP_DELAY);
		this.requestedShippingDate = earlyShipDate.max(requestedShippingDate);
		
		CalculationCalendar earlyCancelDate = this.requestedShippingDate.incrementedBy(CalculationCalendar.DATE, MIN_CANCEL_DELAY);
		this.cancelDate = earlyCancelDate.max(cancelDate);
		allOrders.put(key, this);
	}
	
	public Object newVersion(Object information){return null; }
	
    /** Following methods are used to manipulate the list of orders in memory
    */
    public static Enumeration orders()
    {	return allOrders.elements();	
    }

    public static final Order remove(String orderID)
    {	return (Order)allOrders.remove(orderID);
    }

    public static final void display(String orderID)
    {Order order = (Order)allOrders.get(orderID);
        if (order != null) order.display();
    }
    
    public static final void displayAll()
    {	Enumeration e = orders();
        while (e.hasMoreElements()) {
            Order order = (Order)e.nextElement();
            order.display();
        }
    }
    
	public Customer customer(){return customer;}
	public Address billingAddress(){return customer.billingAddress();}
	public Address shipAddress(){return shipAddress;}
	public Enumeration orderedItems(){return orderedItems.elements();}
	public CalculationCalendar orderDate(){return orderDate;}
	public CalculationCalendar requestedShippingDate(){return requestedShippingDate;}
	public CalculationCalendar cancelDate(){return cancelDate;}
	public boolean valid(){return validOrder;}
	public String key() {return key;}
	public BillingTerms billingTerms(){return terms;}
	
	public void cancelOrder(){ cancelled = true;}
	
	public boolean isCancelled(){ return cancelled;}
	
	/** return the extended total dollar value of this order
	* @return the total value
	*/
	public double total()
	{	Enumeration items = orderedItems();
		double total = 0.0;
		while(items.hasMoreElements())
		{	OrderItem ordered = (OrderItem)items.nextElement();
			total += ordered.quantity() * ordered.unitPrice();
		}		
		return total;
	}
	
	public void billingTerms(BillingTerms billingTerms)
	{	 terms = billingTerms;
	}		
	
	public void shippingAddress(Address address){shipAddress = address;}
	
	public void display()
	{	System.out.println("\t\t\t------- Order -------");
	    customer.display();
		System.out.println("Order ID :" + key + "\n");
	    		
	    System.out.println("\n\n\tQty\tItem No. " +
	            "\tDescription\tUnit Cost\n");
	            
		Enumeration items = orderedItems();		
		while(items.hasMoreElements())
		{	((OrderItem)items.nextElement()).display();
		}
		
		System.out.println("\n\n");
		terms.display();
		System.out.println("Order Date: " + orderDate.getTime() + "\n");
		
        System.out.println("Customer Shipping information:");
		System.out.println(shipAddress());
		System.out.println(" Ship Date: " + 
		                   requestedShippingDate.getTime());
		System.out.println(" Cancel Date: " + 
		                   cancelDate.getTime());

	    System.out.println("---------------------");
	}
	
	public boolean equals(Object other)
	{	if(!(other instanceof Order))return false;
		return key.equals(((Order)other).key);
	}
	
	public int hashcode()
	{	return key.hashCode()*997;
	}
	
	public String toString() {return "Order with id: " + key;}

	private String                      key;
	private Customer                    customer;
	private Address                     shipAddress;
	private Vector                      orderedItems;       // Vector of OrderItems
	private CalculationCalendar         orderDate;
	private CalculationCalendar         requestedShippingDate;
	private CalculationCalendar         cancelDate;// if null order is still open
	private BillingTerms                terms = BillingTerms.NET_10_DAYS;
	
	private boolean						cancelled = false;
	private boolean						validOrder = true;
	
	private static final int MIN_SHIP_DELAY = 7;
	private static final int MIN_CANCEL_DELAY = 10;
    
   private static Hashtable allOrders = new Hashtable();
	
   public static final void close(ObjectOutputStream out)throws IOException
	{	out.writeLong(IDGenerator.nextID);
		out.writeObject(allOrders);
	}
	public static final void open(ObjectInputStream in) 
	throws IOException, ClassNotFoundException
	{	IDGenerator.nextID = in.readLong();
		allOrders = (Hashtable)in.readObject();
	}

	public static class IDGenerator 
	{	public static String nextID()
	    {	return ID_PREFIX + Long.toString(nextID++);
	    }
	    
	    private static long             nextID = 2001;
	    private final static String     ID_PREFIX = "O";
	}
	
	
}

/* notes shipdate  after order date + 7 
	cancel date after ship date + 10
*/