package orderprocessing;

import java.util.Vector;
import java.util.Enumeration;
import java.util.StringTokenizer;
import java.io.ObjectOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.FileOutputStream;
import java.io.FileInputStream;

/** This class accepts and fulfills orders from customers
*/

public class OrderProcessor
{	/** Create a new OrderProcessor for a given InventoryManager
	*	@param inventory The InventoryManager that will resolve orders.
	* Note that this class implements the Singleton Object (Unique Object) design
	* pattern. 
	*/
	private OrderProcessor(InventoryManager inventory) {this.inventory = inventory;}

	/** Generate an order object from string data sent to it
	* @param customer String with name, address, city... tilde ("~") separated
	* @param shipping The shipping address normally associated with this customer (will change)
	* @param items A Vector of strings each of which is a quantity, itemnumber, price, and 
	* 			description. Space separation between fields. 
	* @param orderDate The date of this order.
	* @param requestedShipDate the date shipment is requested.
	* @param cancelDate the date the order should be cancelled if not yet shipped.
	* @return an Order for these items.
	* Note that this is a Factory Method (pattern) for Order objects
	*/
	public Order newOrder
	(	String customer, 
		String terms, 
		String shipping, 
		Vector items, 
		CalculationCalendar orderDate,
		CalculationCalendar requestedShipDate,
		CalculationCalendar cancelDate
	)
	{	Customer c = genCustomer(customer);
		if(c==null){return null;} //No such customer
		if(items.size() == 0){return null;} // no items ordered
		Enumeration e = items.elements();
		Vector orderItems = new Vector();
		while(e.hasMoreElements())
		{	OrderItem result =  genOrderItem((String)e.nextElement(), orderDate);
			if(result == null){return null;} // no such item
			orderItems.addElement(result);
		}
		Address shippingAddress;
		if(shipping.indexOf("default") >=0)
			shippingAddress = c.shippingAddress();
		else
			shippingAddress = new Address(shipping, "~");
		BillingTerms billingTerms = BillingTerms.NET_10_DAYS;
		if(terms.indexOf("default") >=0) 
			billingTerms = c.billingTerms();
		else
		{	if(terms.indexOf("10") >= 0) billingTerms = BillingTerms.NET_10_DAYS;
			else if(terms.indexOf("30") >= 0) billingTerms = BillingTerms.NET_30_DAYS;
		}
		return new Order(c, shippingAddress, billingTerms, orderItems, orderDate, requestedShipDate, cancelDate);
	}
	
	/** Generate an invoice from an order. 
	* @param order An order previously placed by a customer.
	* @return An Invoice for items that can be shipped.
	*/
	public Invoice generateInvoice(Order order)
	{	Enumeration e = order.orderedItems();
		Vector invoicedItems = new Vector();
		while(e.hasMoreElements())
		{	OrderItem trial =  (OrderItem)e.nextElement();
			OrderItem result = inventory.available(trial);
			if(result == null || result.quantity() <= 0)
				System.out.println(trial + " not available");
			else 
				invoicedItems.addElement(result);
		}
		return new Invoice(order, invoicedItems);
	}// This function is incomplete. It doesn't handle partial ships correctly.
	
	private Customer genCustomer(String customer)// may be null
	{	return Customer.get(customer);			                    
	}
	
	private OrderItem genOrderItem(String orderItem, CalculationCalendar date)
	{	StringTokenizer tok = new StringTokenizer(orderItem, " ");
		System.out.println("--> " + orderItem); // Debug code.
		String itemNumber = tok.nextToken();
		int quantity = Integer.parseInt(tok.nextToken());
		if(quantity <= 0){return null;} // nothing requested
		Product item = Product.get(itemNumber);
		if(item == null){return null;}
		return new OrderItem(quantity, item, date);
	}
	
	public static OrderProcessor orderProcessor = new OrderProcessor(InventoryManager.standardInventoryManager);
	
	private InventoryManager    inventory;
	
	public InventoryManager inventory(){return inventory;}
	
	private static ObjectInputStream in = null;
	public static synchronized void close()
	{	try
		{	in.close(); // just in case. 
		} catch (IOException e){}
		  catch (NullPointerException npe){} 
		
		try
		{	
			ObjectOutputStream out = new ObjectOutputStream(new FileOutputStream("opdb.dat"));
			Order.close(out);
			Customer.close(out);
			Invoice.close(out);
			InventoryManager.close(out);
			Product.close(out);
			out.close();
		}
		catch(IOException e ){System.out.println ("Failed writing database.");}
	}	
	
	public static void open()
	{	try
		{	in = new ObjectInputStream(new FileInputStream("opdb.dat"));
			Order.open(in);
			Customer.open(in);
			Invoice.open(in);
			InventoryManager.open(in);
			Product.open(in);
			// This stream is held open until the program halts or until it calls close.			
		}
		catch(IOException e){System.out.println("Failed restoring database.");}
		catch(ClassNotFoundException ce){System.out.println("Class inconsistency");}
	}
	
	private static void initDatabase()// set the database to "empty" state. 
	{	close();
	}
	
	public static void main(String [] args)// set the database to "empty" state.
	{ initDatabase();
	}
}

/* Note that money is handled naively here. To do a better job with money, see Fowler's 
*  Analysis Patterns.  Another naive solution would use a long (integer) representing pennies
*  (or possibly tenths of pennies...). 
*/

/* Note that writing an "error" message to standard output is NEVER adequate error  handling 
*  in a program of this sort for other than run failures. (See generateInvoice.) 
*/

